#! /usr/bin/env python
'''
    Ruchardt.py
    Eric Ayars
    9/2/20  

    A Python program to capture data from the 
    Ruchardt experiment.
'''

########################################
# 
# Students set these values
#
########################################

# constants
N = 1000                        # number of data points to collect, must be <=1000
frequency = 500                 # sampling frequency, Hz

# Channels to watch
channel1 = "Dev1/ai0"           # x axis on graph

########################################
# 
# And leave the rest alone?
#
########################################

import sys                      # useful system calls

import nidaqmx                  # access to NI interfaces

import numpy as np              # arrays, etc.

# GUI stuff
from PyQt5.QtWidgets import QApplication, QWidget, QPushButton, QVBoxLayout, QHBoxLayout, QFileDialog

# plotting package bits
import matplotlib as mpl
mpl.use('QT5Agg')
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg, NavigationToolbar2QT
from matplotlib.figure import Figure

# plotting OOP stuff stolen from 
# https://www.learnpyqt.com/courses/graphics-plotting/plotting-matplotlib/
class MPLCanvas(FigureCanvasQTAgg):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        super(MPLCanvas, self).__init__(fig)

# functions
def start_clicked():
    # measure a set of data points
    global data
    # disable start button
    startButton.setDisabled(True)
    # get stream of data points
    with nidaqmx.Task() as task:
        # set channel to read
        task.ai_channels.add_ai_voltage_chan(channel1)
        # set sampling frequency
        task.timing.cfg_samp_clk_timing(frequency)
        #task.triggering.start_trigger.StartTrigger() #there ought to be a way...
        # collect N points
        data = task.read(number_of_samples_per_channel=N)
    # clear graph
    scene.axes.cla()                
    # redraw graph
    scene.axes.plot(time, data, 'r-')
    scene.axes.set_xlabel("Time (s)")
    scene.axes.set_ylabel("Voltage (V)")
    # show resulting graph
    scene.draw()
    # reactivate buttons appropriately
    startButton.setDisabled(False)
    saveButton.setText("Save Data")
    saveButton.setDisabled(False)

def save_clicked():
    # open datafile
    filename, _ = QFileDialog.getSaveFileName()
    file = open(filename, 'w')
    # write file header
    file.write("# Time\tVoltage\n")
    # write all data (up to N)
    for j in range(N):
        file.write("%0.5f\t%0.5f\n" % (time[j], data[j]))
    # close file
    file.close()
    # update save button text and availability
    saveButton.setText("Data Saved")
    saveButton.setDisabled(True)


#
# Main program
#

data = np.zeros(N)
time = np.array([(1.0/frequency)*dt for dt in range(N)])

app = QApplication(sys.argv)
widget = QWidget()

# scene (graph)
scene = MPLCanvas(width=5, height=4, dpi=100)
scene.axes.set_xlabel("Time (s)")
scene.axes.set_ylabel("Voltage (V)")

# start/save buttons
controls = QHBoxLayout()

startButton = QPushButton(widget)
startButton.setText("Start")
startButton.clicked.connect(start_clicked)

saveButton = QPushButton(widget)
saveButton.setText("Save Data")
saveButton.clicked.connect(save_clicked)
saveButton.setDisabled(True)

controls.addWidget(startButton)
controls.addWidget(saveButton)
controls.addStretch(1)

# graph toolbar
toolbar = NavigationToolbar2QT(scene, widget)

# window layout
layout = QVBoxLayout()
layout.addLayout(controls)
layout.addWidget(toolbar)
layout.addWidget(scene)

# Draw the screen
widget.setLayout(layout)
widget.setWindowTitle("Ruchardt Experiment")
widget.show()

sys.exit(app.exec_())
